/*=====================================================================
File: BottlesPreview.cs

Summary: 

Date : Dc 2006

Origin: Open Cellar ! Project

Revision:

Aut: Matthieu DUCROCQ

---------------------------------------------------------------------
Copyright (C) N/A
=====================================================================*/
using System;
using System.Text;
using System.Drawing;
using System.Drawing.Drawing2D;
using OpenCellar.Framework.Core;
using OpenCellar.Application;

namespace OpenCellar.CustomPreview2
{
	/// <summary>
	/// BottlesPreview.
	/// </summary>
	public class BottlesPreview : DefaultWinePreview
	{
		private Wine m_wine  = null;

		private System.Drawing.Image m_wineImage = null;

		static Size SIZE = new Size(270, 116);

		/// <summary>
		/// Initialise une instance de classe ImagePreview.
		/// </summary>
		public BottlesPreview() : base("Etiquette et nombre de bouteilles", SIZE.Width, SIZE.Height){}

		~BottlesPreview()
		{
			ClearBitmap();
		}

		private void ClearBitmap()
		{
			if(m_wineImage != null)
			{
				m_wineImage.Dispose();
				m_wineImage = null;
			}
		}

		/// <summary>
		/// Rend la previsualisation.
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="item"></param>
		public override void Draw(System.Drawing.Graphics graphics, OpenCellar.Framework.Core.RackItem item)
		{
			CacheBitmap(item.Wine);

			Bitmap b = new Bitmap(SIZE.Width, SIZE.Height);
			Graphics g = Graphics.FromImage(b);
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.Clear(Color.WhiteSmoke);

			//
			// Image
			//
			DrawThumb(g, new Rectangle(10, 35, 60, 60));

			//
			// Infos
			//
			Rectangle bounds = new Rectangle(0, 0, SIZE.Width, SIZE.Height);
			bounds.Width--;
			bounds.Height--;

			//
			// Bordure
			//
			using(Pen borderPen = new Pen(Color.Black))
			{
				g.DrawRectangle(borderPen, bounds);
			}

			//
			// Infos
			//
			DrawInfos(g, bounds);

			graphics.DrawImageUnscaled(b, 0, 0);
			b.Dispose();
			g.Dispose();
		}

		/// <summary>
		/// Dfinit la source de donnes su controle.
		/// </summary>
		/// <param name="wine"></param>
		private void CacheBitmap(Wine wine)
		{
			if(m_wine != wine)
			{
				ClearBitmap();
	
				m_wine = wine;

				//
				// A t-il une image associe
				//
				if(m_wine.HasImage())
				{
					OpenCellar.Framework.Core.Image i = wine.Image;
					if(i != null)
					{
						try
						{
							//
							// Calcul le ratio
							//
							m_wineImage = i.GetImage();
							if((m_wineImage.Width > 60) | (m_wineImage.Height > 60))
							{
								if(m_wineImage.Width > m_wineImage.Height)
								{
									Bitmap tempImage = new Bitmap(m_wineImage, 60, (int)(60 * (float)m_wineImage.Height / m_wineImage.Width));
									m_wineImage = tempImage;
								}
								else if(m_wineImage.Width < m_wineImage.Height)
								{
									Bitmap tempImage = new Bitmap(m_wineImage, (int)(60 * (float)m_wineImage.Width / m_wineImage.Height), 60);
									m_wineImage = tempImage;
								}
								else
								{
									Bitmap tempImage = new Bitmap(m_wineImage, 60, 60);
									m_wineImage = tempImage; 
								}
							}
						}
						catch
						{
							//
							// zZzzZ
							//
							ClearBitmap();
						}
						finally
						{
							if(i != null)
							{
								i.Dispose();
								i = null;
							}
						}
					}					
				}				
			}
		}

		/// <summary>
		/// Dessine l'etiquette du vin.
		/// </summary>
		/// <param name="g"></param>
		/// <param name="bounds"></param>
		private void DrawThumb(Graphics g, Rectangle bounds)
		{
			//
			// On recupere l'image
			//
			if(m_wineImage != null)
			{
				
				if((m_wineImage.Width == 60) && (m_wineImage.Height < 60))
				{
					g.DrawImage(m_wineImage, bounds.X, bounds.Y + ((bounds.Height - m_wineImage.Height) / 2));
				}
				else if((m_wineImage.Width == 60) && (m_wineImage.Height == 60))
				{
					g.DrawImage(m_wineImage, bounds.X, bounds.Y);
				}
				else if((m_wineImage.Width < 60) && (m_wineImage.Height == 60))
				{
					g.DrawImage(m_wineImage, bounds.X + ((bounds.Width - m_wineImage.Width) / 2), bounds.Y);
				}
				else
				{
					g.DrawImage(m_wineImage, bounds.X + ((bounds.Width - m_wineImage.Width) / 2), bounds.Y + ((bounds.Height - m_wineImage.Height) / 2));
				}
			}

			//
			// Contour
			//
			using (Pen p = new Pen(Color.Black))
			{
				g.DrawRectangle(p, bounds);
			}
		}

		/// <summary>
		/// Dessine les information du vin.
		/// </summary>
		/// <param name="g">Contexte graphique</param>
		/// <param name="bounds">Rectangle de destination.</param>
		private void DrawInfos(Graphics g, Rectangle bounds)
		{
			bounds.Width-=1;
			bounds.Height-=1;
			Rectangle title = new Rectangle(new Point(bounds.X, bounds.Y), new Size(bounds.Width, 32));
			using(SolidBrush sb = new SolidBrush(Color.Black))
			{
				using(Pen p = new Pen(Color.Black))
				{
					g.DrawLine(p, new Point(0, 32), new Point(bounds.Width, 32));
				}
				
				using(LinearGradientBrush lgb = new LinearGradientBrush(title, Utility.GetWineColor(m_wine), Color.WhiteSmoke, LinearGradientMode.Vertical))
				{
					g.FillRectangle(lgb, title);
			
					string titleText = string.Empty;
					if(m_wine.Year != 0)
					{
						titleText += m_wine.Year.ToString() + " - ";
					}

					if(m_wine.Name.Trim() != string.Empty)
					{
						titleText += m_wine.Name.Trim();
					}
					else
					{
						titleText += "Vin sans nom";
					}

					//
					// Nombre de bouteilles
					//
					if(m_wine.ManualManagement)
					{
						titleText+= " (" + m_wine.Bottles.ToString() + ")";
					}
					else
					{
						titleText+= " (" + m_wine.RackItems.Count.ToString() + ")";
					}
			
					title.X+=4;
					title.Width-=4;
					g.DrawString(titleText, this.TitleFont, sb, title, stringFormat);

					title = new Rectangle(bounds.X + 74, bounds.Y + 35, bounds.Width - 74, 16);
					StringBuilder stringBuilder = new StringBuilder();
					
					//
					// Pays
					//
					string tmp = m_wine.Country.Name.Trim();

					if((tmp != null) && (tmp != string.Empty))
					{
						stringBuilder.Append(tmp + " - ");
					}

					//
					// Region
					//
					tmp = m_wine.Area.Name.Trim();

					if((tmp != null) && (tmp != string.Empty))
					{
						stringBuilder.Append(tmp + " - ");
					}

					//
					// Cepage
					//
					tmp = m_wine.Cepage.Name.Trim();

					if((tmp != null) && (tmp != string.Empty))
					{
						stringBuilder.Append(tmp + " - ");
					}

					tmp = stringBuilder.ToString().Trim();
					if(tmp.EndsWith("-"))
					{
						tmp = tmp.Substring(0, tmp.Length - 1);
					}

					g.DrawString(tmp, this.Font, sb, title, stringFormat);

					//
					// Appellation
					//
					title = new Rectangle(bounds.X + 74, bounds.Y + 52, bounds.Width - 74, 16);
					g.DrawString(m_wine.Appellation.Name.Trim(), this.Font, sb, title, stringFormat);

 
					//
					// A boire et apoge
					//
					title = new Rectangle(bounds.X + 74, bounds.Y + 70, bounds.Width - 74, 16);
					tmp = string.Empty;
					if((m_wine.ConsumeMin > 0) && (m_wine.ConsumeMax > 0))
					{
						tmp = "Bo. " + m_wine.ConsumeMin.ToString() + "  " + m_wine.ConsumeMax.ToString();
					}

					if((m_wine.BestMin > 0) && (m_wine.BestMax > 0))
					{
						if(tmp != string.Empty)
						{
							tmp+= " ";
						}
						tmp += "Ap. " + m_wine.BestMin.ToString() + "  " + m_wine.BestMax.ToString();
					}
					if(tmp != string.Empty)
					{
						g.DrawString(tmp, this.Font, sb, title, stringFormat);
					}
				}
			}
		}
	}
}
